

/********************************************************************************
 * 02/011/2013
 * LCD G2452 I2C
 * E un software sviluppato con il 430G2452,ma pu essere facilmente adattato a
 * qualsiesi 430G2xx2 salvo modificare le due porte P2.3 e P2.4 collegate a RS
 * e E dell'LCD. La trasmissione del dati all'LCD avviene via porta I2C connessa
 * a un expander PCF8574A alimentato a 3.3v, la gestione dell'LCD  di conseguenza
 * a 8 bit. L'LCD  alimentato a 5V e si pu connettere LCD del tipo 1x16,2x16 e
 * 4x16.
 * Il clock della CPU  8MHz quello ausiliario per I2C  SMCLK = MCLK/64(USIDIV_6)
 * par a ~125KHz
 * ACLK = oscillatore esterno non  previsto
 * L'indirizzo di chiamata dello slave deve corrispondere al cablato sul PCF8574A
 * Lo schema di collegamento ha lo stesso titolo LCD-I2C-G2252
 *
 */

/*
 * le costanti sottoelencate servono solo da test
 * unsigned int da_canc = 65432;
 * const char STR1[] = "123456 prova LCD";
 * 1) si scrive in pos1 riga1 "01234"
 * 2) si scrive in po1 riga2 "123456 prova LCD"
 * 3) si scrive in pos8 riga1 "65432"
 * 4) si cancella la seconda riga
 * 5) si scrive  in pos3 riga 2 "HALLO WORLD"
 * 6) viene poi attivato ADC e si legge il valore del potenziometro su P1.1
 */



#include <msp430.h> 
#include "LCD430.h"


// Global Variable declarations
char flag;				//per uscire dal loop in I2C
char i,x;
char slv_addr = 0X70;   // indirizzo di chiamata dello slave
char disp[16]; 			// Una riga di Buffer per caratteri da inviare a LCD
char MST_data;
int I2C_State = 0;
unsigned int da_canc = 65432;					   // per prova da cancellare
const char STR1[] = "123456 prova LCD";



/*
 * main.c
 */
int main(void) {
    WDTCTL = WDTPW | WDTHOLD;	           // Stop watchdog timer
	  if (CALBC1_8MHZ==0xFF)			   // If calibration constants erased
	  {
	    while(1);                          // do not load, trap CPU!!
	  }
	  DCOCTL = 0;                          // Select lowest DCOx and MODx settings
	  BCSCTL1 = CALBC1_8MHZ;               // Set DCO
	  DCOCTL = CALDCO_8MHZ;

	  P1OUT = 0xC0;                        // P1.6 & P1.7 Pullups, altre a  0
	  P1REN |= 0xC0;                       // P1.6 & P1.7 Pullups
	  P1DIR = 0xFF;                        // Pin non usati settati ein uscita
	  P2OUT = 0;
	  P2DIR = 0xFF;

	  USICTL0 = USIPE6+USIPE7+USIMST+USISWRST; // Port & USI in modo setup
	  USICTL1 = USII2C+USIIE;                  // abilita I2C & USI interrupt
	  USICKCTL = USIDIV_7+USISSEL_2+USICKPL;  //Setup USI clocks: SCL=SMCLK/16 =750k
	  USICNT |= USIIFGCC;                      // Disabilita automatic clear control
	  USICTL0 &= ~USISWRST;                    // abilita USI
	  USICTL1 &= ~USIIFG;                      // Clear pending flag

	  /*
	   * configuro ADC
	   * SREF_1       	Converte tra la tensione di alimentazione e massa
	   * ADC10SHT_3   	sample e hold time 64 ADC clocks
	   * ADC10ON		ADC10 ON
	   * ADC10IE		abilita interrupt di ADC
	   */
	  ADC10CTL0 = SREF_0 + ADC10SHT_3 + ADC10ON + ADC10IE;
	  ADC10CTL1 = INCH_1;              // l'ingresso  A1
	  ADC10AE0 |= 0x02;                // la porta PA.1  l'ingresso ADC

	  _enable_interrupt();					   // Abilita interrupt

  init_LCD();				//inizializza LCD

  LCD_pos_line(1,1);
  for(i=0;i<5; i++){		//manda a LCD il numero
  LCD_wrt(1,(0x30+i));
  }


  LCD_pos_line(1,2);
  LCD_string(STR1);

  LCD_pos_line(8,1);
  int_to_str(da_canc);
  LCD_string(disp);


  clr_line_buffer();
  LCD_pos_line(1,2);
  LCD_string(disp);

  LCD_pos_line(3,2);
  LCD_string("HELLO WORD");


  /*
   * faccio ora una prova con il convertitore analogico digitale
   */

  LCD_clear();							// cancello il display

  LCD_pos_line(1,1);    				// posizione 1 riga 1
  LCD_string(" NUMERO SCALINI ");


  for (;;) {
    ADC10CTL0 |= ENC + ADC10SC;      	// Sampling and conversion start
    LPM0;       						// LPM0, ADC10_ISR forzer l'uscita
    _NOP();								// Per il debug
    int_to_str(ADC10MEM);				// il risultato vdella conversione in stringa
    LCD_pos_line(5,2);                  // posizione 5 riga 2
    LCD_string(disp) ;					// a display il risultato della conversione
  }



	return 0;
}

/******************************************************
// USI interrupt service routine
******************************************************/
#pragma vector = USI_VECTOR
__interrupt void USI_TXRX (void)
{
  switch(I2C_State)
    {
      case 0: // Genera lo  Start e invia l'indirizzo
              P1OUT |= 0x01;          // LED on: sequence start
              USISRL = 0x00;           	// Genera LO  Start .....
              USICTL0 |= USIGE+USIOE;
              USICTL0 &= ~USIGE;
              USISRL = slv_addr;       	// ... e trasmette l'indirizzo dello slave
              USICNT = (USICNT & 0xE0) + 0x08; // Bit counter = 8, TX Address
              I2C_State = 2;           	// allo stato successivo: receive address (N)Ack
              break;

      case 2: // Receve Address Ack/Nack bit
              USICTL0 &= ~USIOE;       // SDA = input
              USICNT |= 0x01;          // Bit counter = 1, receve (N)Ack bit
              I2C_State = 4;           // allo stato successivo: check (N)Ack
              break;

      case 4: // Processa  Address Ack/Nack & trasmette  MST_data
              USICTL0 |= USIOE;        // SDA = output
              if (USISRL & 0x01)       // Se riceve un  Nack ...
              { // Trasmette uno  stop...
                USISRL = 0x00;
                USICNT |=  0x01;       // Bit counter = 1, SCL high, SDA low
                I2C_State = 10;        // Va allo stato successivo: generate Stop
                P1OUT |= 0x01;         // Accende il LED: errore
              }
              else
              { // Se ricevuto un Ack, TX data allo slave...
                USISRL = MST_data;     // carica il data byte
                USICNT |=  0x08;       // Bit counter = 8, inizia TX
                I2C_State = 6;         // va allo stato successivo: receive data (N)Ack
                P1OUT &= ~0x01;      // spegne il LED
              }
              break;

      case 6: // Receve il bit Ack/Nack
              USICTL0 &= ~USIOE;       	// SDA = input
              USICNT |= 0x01;          	// Bit counter = 1, receve (N)Ack bit
              I2C_State = 8;           	// va allo stato successivo: verisica se  (N)Ack
              break;

      case 8: // Processa MST_data in funzione di  Ack/Nack & invio lo Stop
              USICTL0 |= USIOE;
              if (USISRL & 0x01)       		// se ha ricevuto un Nack ...
                P1OUT |= 0x01;         		// accende il LED: errore
              else                     		// se  invece un  Ack
              {
                MST_data++;            		// Incrementa il Master data
                P1OUT &= ~0x01;        	// spegne il LED
              }
              // Send stop...
              USISRL = 0x00;
              USICNT |=  0x01;         		// Bit counter = 1, SCL high, SDA low
              I2C_State = 10;          		// va allo stato successivo: genera uno Stop
              break;

      case 10:// Genera la condizione di Stop
              USISRL = 0x0FF;          		// USISRL = 1 to release SDA
              USICTL0 |= USIGE;        		// Abilitato il Transparent latch
              USICTL0 &= ~(USIGE+USIOE);	//  Disabilita Latch/SDA output
              I2C_State = 0;           		// Resetta lo stato per una nuova trasmissione
              LPM0_EXIT;               		// stato attivo per il nuovo transfer
              flag=1;
              break;
    }

  USICTL1 &= ~USIIFG;                  // Clear il flag attivo
}

#pragma vector=TIMER0_A0_VECTOR
__interrupt void ta0_isr(void)
{
  TACTL = 0;
  LPM0_EXIT;                                // Exit LPM0 on return
}

// ADC10 interrupt service routine
#pragma vector=ADC10_VECTOR
__interrupt void ADC10_ISR (void)
{
	LPM0_EXIT;

}

